// Copyright 2023 Michael Amann and contributors
// SPDX-License-Identifier: AGPL-3.0-only

package configuration

import (
	"fmt"
	"os"

	"git.kle.li/tools/go-import-redirector/pkg/configuration/file"
	"git.kle.li/tools/go-import-redirector/pkg/packagelist"
	"github.com/fsnotify/fsnotify"
	"github.com/spf13/viper"
)

type Configuration struct {
	v           *viper.Viper
	r           func()
	packageList *packagelist.PackageList
}

func New(cfgFile string, hotReload bool, reloadFunction func()) *Configuration {
	c := Configuration{
		v: viper.New(),
	}

	c.v.Debug()

	if cfgFile != "" {
		// Use Configurationfile from the flag.
		c.v.SetConfigFile(cfgFile)
	} else {
		// Find home directory.
		home, err := os.UserHomeDir()
		if err == nil {
			c.v.AddConfigPath(home)
		} else {
			fmt.Println("Unable to determine $HOME, not searching it for potential config")
		}

		c.v.AddConfigPath("/etc/")
		c.v.AddConfigPath(".")
		c.v.SetConfigType("yaml")
		c.v.SetConfigName("go-import-redirector")
	}

	// If a Configurationfile is found, read it in.
	if err := c.v.ReadInConfig(); err == nil {
		fmt.Fprintln(os.Stderr, "Using Configurationfile:", c.v.ConfigFileUsed())
	} else {
		fmt.Fprintf(os.Stderr, "Failed to load any configs %v", err.Error())
		os.Exit(1)
	}

	if hotReload {
		c.r = reloadFunction
		fmt.Fprintln(os.Stderr, "Hot reloading enabled")
		c.v.OnConfigChange(c.updateConfig)
		c.v.WatchConfig()
	} else {
		fmt.Fprintln(os.Stderr, "Hot reloading disabled")
	}

	c.loadCfgFile()

	return &c
}

func (c *Configuration) updateConfig(_ fsnotify.Event) {
	c.loadCfgFile()

	if c.r != nil {
		c.r()
	}
}

func (c *Configuration) GetPackageList() *packagelist.PackageList {
	return c.packageList
}

func (c *Configuration) loadCfgFile() {
	cfgFile := &file.CfgFile{}
	if err := c.v.Unmarshal(&cfgFile); err != nil {
		fmt.Fprintf(os.Stderr, "Failed to unmarshal config %v", err.Error())
		os.Exit(1)
	}

	if c.v.GetString("host") == "" {
		fmt.Fprintln(os.Stderr, "Host not set in config")
		os.Exit(1)
	}

	if cfgFile.Modules == nil || len(cfgFile.Modules) == 0 {
		fmt.Fprintln(os.Stderr, "No modules in config")
		os.Exit(1)
	}

	cfgFile.UpdateCfgWithDefaults()

	c.packageList = packagelist.NewPackageList(cfgFile.Host, cfgFile.Modules)
}
