// Copyright 2023 Michael Amann and contributors
// SPDX-License-Identifier: AGPL-3.0-only

package server

import (
	"context"
	"log"
	"net/http"
	"sync"
	"time"

	"git.kle.li/tools/go-import-redirector/pkg/packagelist"
)

type Server struct {
	srv  *http.Server
	wg   *sync.WaitGroup
	bind string
	pkgs *packagelist.PackageList
}

func New(bind string) *Server {
	s := &Server{
		wg:   &sync.WaitGroup{},
		bind: bind,
	}
	return s
}

func (s *Server) startHTTPServer() {
	s.srv = &http.Server{Addr: s.bind}
	http.DefaultServeMux = new(http.ServeMux)
	http.HandleFunc("/", s.pkgs.ServeHTTP)
	go func() {
		defer s.wg.Done() // let main know we are done cleaning up

		// always returns error. ErrServerClosed on graceful close
		if err := s.srv.ListenAndServe(); err != http.ErrServerClosed {
			// unexpected error. port in use?
			log.Fatalf("ListenAndServe(): %v", err)
		}
	}()
}

func (s *Server) Restart(pkgs *packagelist.PackageList) {
	s.Shutdown()
	s.Start(pkgs)
}

func (s *Server) Start(pkgs *packagelist.PackageList) {
	s.pkgs = pkgs
	s.wg.Add(1)
	s.startHTTPServer()
}

func (s *Server) Shutdown() {
	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	defer cancel()
	if err := s.srv.Shutdown(ctx); err != nil {
		panic(err) // failure/timeout shutting down the server gracefully
	}
	s.wg.Wait()
}
